/*
 * Copyright 2007 Ask Hjorth Larsen
 *
 * asklarsen@gmail.com
 *
 * This file is part of Wires.
 * 
 * Wires is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wires is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Wires.  If not, see <http://www.gnu.org/licenses/>.
 */

package jwires;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author ask
 */
public class Board extends JPanel {
    
    //public final static int EDGE_SIZE = 2;
    
    public final static Color
            lockedColor = new Color(128, 128, 128, 96),
            monoValentColor = new Color(0, 150, 130),
            filledColor = new Color(0, 100, 80),
            wireColor = Color.BLACK,
            floodTileColor = new Color(255, 0, 0, 192);

    private final TileMap map;
    private int tileSize = 60;    

    private Tile floodTile;
    //private int floodFillX, floodFillY;
    private boolean floodFilled;
    
    /** Creates a new instance of Board */
    public Board(TileMap map) {
        this.map = map;
        
        setBackground(Color.white);
        Listener listener = new Listener();
        addMouseListener(listener);
        addMouseWheelListener(listener);
        setPreferredSize(new Dimension(tileSize*map.width /*+ 2*EDGE_SIZE*/, tileSize*map.height /*+ 2*EDGE_SIZE*/));
    }
    
    protected void paintComponent(Graphics g){
        super.paintComponent(g);
        ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        //g.translate(EDGE_SIZE, EDGE_SIZE);
        // replace with iteration over neighbours or something
        // ... or maybe that is not necessary.
        for(int i = 0; i < map.width; i++){
            for(int j = 0; j < map.height; j++){
                drawTile(map.getTile(i, j), g, i*tileSize, j*tileSize);
            }
        }
        //g.translate(-EDGE_SIZE, -EDGE_SIZE);
    }
    
    public void drawFloodBlob(Graphics g, int x, int y){
        if(floodFilled){   
            //int x = floodFillX * tileSize;
            //int y = floodFillY * tileSize;
            g.setColor(floodTileColor);
            int size = tileSize / 3;
            g.fillOval(x+tileSize/2-size/2, y+tileSize/2-size/2, size, size);
        }        
    }
    
    /*public void paintEdge(Graphics g){
        Tile tile;
        for(int i = 0; i < map.width; i++){
            tile = map.getTile(i, 0);
            if(tile.isConnected(1)){
                
            }
        }
        for(int i = 0; i < map.height; i++){
            
        }
    }*/
    
    public void shift(int direction){
        map.shift(direction);
        /*int dx=0, dy=0;
        switch(direction){
            case 0:
                dx = 1;
                break;
            case 1:
                dy = 1;
                break;
            case 2:
                dx = -1;
                break;
            case 3:
                dy = -1;
                break;
        }
        floodFillX += dx;
        floodFillY += dy;*/
    }
    
    public void drawTile(Tile tile, Graphics g, int x, int y){
        if(tile.isLocked()){
            g.setColor(lockedColor);
            g.fillRect(x, y, tileSize, tileSize);
        }
        
        int chunk = tileSize/5;
        if(tile.getConnectionCount() == 1){
            g.setColor(monoValentColor);
            g.fillRect(x+chunk, y+chunk, 3*chunk, 3*chunk);
        }
        
        g.setColor(tile.isMarked() ? filledColor : wireColor);
        if(tile.isConnected(0)){
            g.fillRect(x+2*chunk, y+2*chunk, 3*chunk, 1*chunk);
        }
        if(tile.isConnected(1)){
            g.fillRect(x+2*chunk, y+0*chunk, 1*chunk, 3*chunk);
        }
        if(tile.isConnected(2)){
            g.fillRect(x+0*chunk, y+2*chunk, 3*chunk, 1*chunk);
        }
        if(tile.isConnected(3)){
            g.fillRect(x+2*chunk, y+2*chunk, 1*chunk, 3*chunk);
        }
        if(tile == floodTile){
            drawFloodBlob(g, x, y);
        }
        /*if(tile.isMarked()){
            int d = chunk/3;
            g.setColor(Color.red);
            if(tile.isConnected(0)){
                g.fillRect(x + 2*chunk + d, y + 2*chunk + d, 3*chunk - d, d);
            }
            if(tile.isConnected(1)){
                g.fillRect(x + 2*chunk + d, y, d, 3*chunk - d);
            }
            if(tile.isConnected(2)){
                g.fillRect(x, y + 2*chunk + d, 3*chunk - d, d);
            }
            if(tile.isConnected(3)){
                g.fillRect(x + 2*chunk + d, y + 2*chunk + d, d, 3*chunk - d);
            }
        }*/
    }

    public void floodFill(int x, int y){
        this.floodTile = map.getTile(x, y);
        /*this.floodFillX = x;
        this.floodFillY = y;*/
        floodFill();
    }
    
    public void floodFill(){
        if(floodFilled){
            map.clearMarks();
            //floodFilled = false;
        } //else {
        floodFilled = true;
        int connectedCount = map.floodFill(floodTile.getX(), floodTile.getY());
        checkWin(connectedCount);
    }

    public void checkWin(int connectedCount){
        if(connectedCount == map.width * map.height){
            // All tiles connected. We have a winner.
            for(Tile tile : map.getTiles()){
                tile.setLocked(true);
            }
            Board.this.repaint();
            JOptionPane.showMessageDialog(null, "Congratulations!", "Game over", 
                    JOptionPane.WARNING_MESSAGE);
        }        
    }
    
    private class Listener implements MouseListener, MouseWheelListener {
        public void mouseClicked(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
            int button = e.getButton();
            int x = map.width * e.getX()/(tileSize*map.width);
            int y = map.height * e.getY()/(tileSize*map.height);
            if(button == e.BUTTON1){
                map.getTile(x, y).toggleLock();
                repaint();
            } else if(button == e.BUTTON3){
                if(floodFilled){
                    map.clearMarks();
                    floodFilled = false;
                    repaint();
                } else {
                    floodFill(x, y);
                    repaint();
                }
            }
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseWheelMoved(MouseWheelEvent e) {
            int rotation = e.getWheelRotation();
            int x = map.width * e.getX()/(tileSize*map.width);
            int y = map.height * e.getY()/(tileSize*map.height);
            Tile tile = map.getTile(x, y);
            if(!tile.isLocked()){
                map.getTile(x,y).rotate(rotation);
                if(floodFilled){
                    /*boolean reFloodFill = false;
                    for(int i = 0; i < 4; i++){
                        Tile neighbour = tile.getNeighbour(i);
                        if(neighbour.isMarked()){
                            reFloodFill = true;
                            break;
                        }
                    }*/
                    floodFill();
                    /*if(reFloodFill){
                        floodFill();
                    }*/
                    
                }
            }
            repaint();
        }
        
    }
    
}
